﻿using anbs_cp.Interfaces;

namespace anbs_cp.Classes;

/// <summary>
/// Класс ошибки
/// </summary>
public sealed class ActionError: IActionError
{
    /// <summary>
    /// Критичность ошибки:
    /// при некритичных ошибках продолжение выполнения операции возможно,
    /// а при критичных -- нет
    /// </summary>
    public bool IsCritical { get; init; }

    /// <summary>
    /// Объект ошибки
    /// </summary>
    public object Object { get; set; }

    /// <summary>
    /// Сообщение об ошибке
    /// </summary>
    public string Message { get; set; }

    #region Конструкторы
    /// <summary>
    /// Конструктор по умолчанию
    /// </summary>
    public ActionError ()
    {
        IsCritical = true;
        Object = "Not Implemented";
        Message = "Not Implemented";
    }

    /// <summary>
    /// Конструктор с 2 параметрами
    /// </summary>
    /// <param name="message">Сообщение</param>
    /// <param name="isCritical">Критичность ошибки</param>
    public ActionError (string message, bool isCritical = true)
    {
        IsCritical = isCritical;
        Object = "";
        Message = message;
    }

    /// <summary>
    /// Конструктор с 3 параметрами
    /// </summary>
    /// <param name="eObject">Объект ошибки</param>
    /// <param name="message">Сообщение</param>
    /// <param name="isCritical">Критичность ошибки</param>
    public ActionError (object eObject, string message, bool isCritical = true)
    {
        IsCritical = isCritical;
        Object = eObject;
        Message = message;
    }
    #endregion
}