﻿using anbs_cp.Interfaces;

namespace anbs_cp.Classes;

/// <summary>
///     Форматирует размер файла/папки в понятную строку
/// </summary>
public class FileSizeFormatter : IValueFormatter
{
    #region Cвойства класса

    /// <summary>
    ///     Имена размеров (байт, килобайт, мегабайт, гигабайт и террабайт)
    /// </summary>
    public string[] SizeNames { get; set; } = { "Байт", "Кб", "Мб", "Гб", "Тб" };

    /// <summary>
    ///     Знаков после запятой
    /// </summary>
    public byte DecimalPlaces { get; set; } = 2;

    /// <summary>
    ///     Максимально байт (далее идут Кбайты)
    /// </summary>
    public long ByteMax { get; set; } = 1024;

    /// <summary>
    ///     Максимально Кбайт (далее идут Мбайты)
    /// </summary>
    public long KByteMax { get; set; } = 1048576;

    /// <summary>
    ///     Максимально Мбайт (далее идут Гбайты)
    /// </summary>
    public long MByteMax { get; set; } = 1073741824;

    /// <summary>
    ///     Максимально Гбайт (далее идут Тбайты)
    /// </summary>
    public long GByteMax { get; set; } = 1099511627776;

    #endregion

    #region Реализация интерфейса

    /// <summary>
    ///     Реализация интерфейса
    /// </summary>
    public string[] ValueNames
    {
        get => SizeNames;
        set => SizeNames = value;
    }

    /// <summary>
    ///     Реализация интерфейса
    /// </summary>
    public long[] MaxSizes
    {
        get => new[] { ByteMax, KByteMax, MByteMax, GByteMax };
        set
        {
            ByteMax = value[0];
            KByteMax = value[1];
            MByteMax = value[2];
            GByteMax = value[3];
        }
    }

    #endregion
}