﻿using System.Globalization;

using Newtonsoft.Json;

namespace anbs_cp.Classes;

/// <summary>
/// Конвертер типов на манер Delphi
/// </summary>
public static class TypeConverter
{
    #region Конвертация числа в строку

    /// <summary>
    /// Преобразование <see cref="int"/> в <see cref="string"/>
    /// </summary>
    /// <param name="value">Число</param>
    /// <returns>Значение в <see cref="string"/></returns>
    public static string IntToStr (int value) => value.ToString();

    /// <summary>
    /// Преобразование <see cref="uint"/> в <see cref="string"/>
    /// </summary>
    /// <param name="value">Число</param>
    /// <returns>Значение в <see cref="string"/></returns>
    public static string IntToStr (uint value) => value.ToString();

    /// <summary>
    /// Преобразование <see cref="long"/> в <see cref="string"/>
    /// </summary>
    /// <param name="value">Число</param>
    /// <returns>Значение в <see cref="string"/></returns>
    public static string IntToStr (long value) => value.ToString();

    /// <summary>
    /// Преобразование <see cref="ulong"/> в <see cref="string"/>
    /// </summary>
    /// <param name="value">Число</param>
    /// <returns>Значение в <see cref="string"/></returns>
    public static string IntToStr (ulong value) => value.ToString();

    /// <summary>
    /// Преобразование <see cref="byte"/> в <see cref="string"/>
    /// </summary>
    /// <param name="value">Число</param>
    /// <returns>Значение в <see cref="string"/></returns>
    public static string IntToStr (byte value) => value.ToString();

    /// <summary>
    /// Преобразование <see cref="decimal"/> в <see cref="string"/>
    /// </summary>
    /// <param name="value">Число</param>
    /// <returns>Значение в <see cref="string"/></returns>
    public static string DecimalToStr (decimal value) => value.ToString(CultureInfo.InvariantCulture);

    /// <summary>
    /// Преобразование <see cref="double"/> в <see cref="string"/>
    /// </summary>
    /// <param name="value">Число</param>
    /// <returns>Значение в <see cref="string"/></returns>
    public static string DoubleToStr (double value) => value.ToString(CultureInfo.InvariantCulture);

    /// <summary>
    /// Преобразование <see cref="bool"/> в <see cref="string"/>
    /// </summary>
    /// <param name="value">Значение правда/ложь</param>
    /// <returns>Значение в <see cref="string"/></returns>
    public static string BoolToStr (bool value) => value.ToString();

    /// <summary>
    /// Преобразование любого типа в <see cref="string"/> (сериализация)
    /// </summary>
    /// <typeparam name="T">Тип</typeparam>
    /// <param name="value">Значение типа</param>
    /// <returns>Значение в <see cref="string"/></returns>
    public static string TypeToStr<T> (T value) => JsonConvert.SerializeObject(value);

    #endregion

    #region Конвертация строки в число

    /// <summary>
    /// Преобразование <see cref="string"/> в <see cref="int"/>
    /// </summary>
    /// <param name="value">Строка</param>
    /// <param name="defaultValue">Значение по умолчанию</param>
    /// <returns>Значение в <see cref="int"/></returns>
    public static int StrToInt (string value, int defaultValue = 0) =>
        int.TryParse(value, out int result) ? result : defaultValue;

    /// <summary>
    /// Преобразование <see cref="string"/> в <see cref="uint"/>
    /// </summary>
    /// <param name="value">Строка</param>
    /// <param name="defaultValue">Значение по умолчанию</param>
    /// <returns>Значение в <see cref="uint"/></returns>
    public static uint StrToUInt (string value, uint defaultValue = 0) =>
        uint.TryParse(value, out uint result) ? result : defaultValue;

    /// <summary>
    /// Преобразование <see cref="string"/> в <see cref="long"/>
    /// </summary>
    /// <param name="value">Строка</param>
    /// <param name="defaultValue">Значение по умолчанию</param>
    /// <returns>Значение в <see cref="long"/></returns>
    public static long StrToInt64 (string value, long defaultValue = 0) =>
        long.TryParse(value, out long result) ? result : defaultValue;

    /// <summary>
    /// Преобразование <see cref="string"/> в <see cref="ulong"/>
    /// </summary>
    /// <param name="value">Строка</param>
    /// <param name="defaultValue">Значение по умолчанию</param>
    /// <returns>Значение в <see cref="ulong"/></returns>
    public static ulong StrToUInt64 (string value, ulong defaultValue = 0) =>
        ulong.TryParse(value, out ulong result) ? result : defaultValue;

    /// <summary>
    /// Преобразование <see cref="string"/> в <see cref="byte"/>
    /// </summary>
    /// <param name="value">Строка</param>
    /// <param name="defaultValue">Значение по умолчанию</param>
    /// <returns>Значение в <see cref="byte"/></returns>
    public static byte StrToByte (string value, byte defaultValue = byte.MinValue) =>
        byte.TryParse(value, out byte result) ? result : defaultValue;

    /// <summary>
    /// Преобразование <see cref="string"/> в <see cref="decimal"/>
    /// </summary>
    /// <param name="value">Строка</param>
    /// <param name="defaultValue">Значение по умолчанию (<see cref="decimal.Zero"/>)</param>
    /// <returns>Значение в <see cref="decimal"/></returns>
    public static decimal StrToDecimal (string value, decimal defaultValue = decimal.Zero) =>
        decimal.TryParse(value, out decimal result) ? result : defaultValue;

    /// <summary>
    /// Преобразование <see cref="string"/> в <see cref="double"/>
    /// </summary>
    /// <param name="value">Строка</param>
    /// <param name="defaultValue">Значение по умолчанию</param>
    /// <returns>Значение в <see cref="double"/></returns>
    public static double StrToDouble (string value, double defaultValue = 0) =>
        double.TryParse(value, out double result) ? result : defaultValue;

    /// <summary>
    /// Преобразование <see cref="string"/> в <see cref="bool"/>
    /// </summary>
    /// <param name="value">Строка</param>
    /// <param name="defaultValue">Значение по умолчанию</param>
    /// <returns>Значение в <see cref="bool"/></returns>
    public static bool StrToBool (string value, bool defaultValue = false) =>
        bool.TryParse(value, out bool result) ? result : defaultValue;

    /// <summary>
    /// Преобразование <see cref="string"/> в тип <see cref="T"/> (десериализация)
    /// </summary>
    /// <typeparam name="T">Тип</typeparam>
    /// <param name="value">Строка</param>
    /// <param name="defaultValue">Значение по умолчанию</param>
    /// <returns>Значение в <see cref="T"/></returns>
    public static T StrToType<T>(string value, T defaultValue) =>
        JsonConvert.DeserializeObject<T>(value) ?? defaultValue;

    #endregion
}