﻿using anbs_cp.Interfaces;

namespace anbs_cp.Classes;

/// <summary>
/// Абстрактный класс конвертера величин для отображения (улучшенный аналог ValueFormatter)
/// </summary>
public abstract class ValueConverter: IValueConverter
{
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="valueNames">Массив имён размерностей</param>
    /// <param name="divider">Делитель</param>
    /// <param name="decimalPlaces">Число знаков после запятой</param>
    protected ValueConverter (string[] valueNames, long divider, byte decimalPlaces)
    {
        ValueNames = valueNames;
        Divider = divider;
        DecimalPlaces = (byte)(decimalPlaces < 3 ? decimalPlaces : 2);
    }

    #region Реализация интерфейса
    /// <summary>
    /// Массив имён размерностей
    /// </summary>
    public string[] ValueNames { get; init; }

    /// <summary>
    /// Делитель
    /// </summary>
    public long Divider { get; init; }

    /// <summary>
    /// Знаков после запятой (0, 1, 2)
    /// </summary>
    public byte DecimalPlaces { get; init; }

    #endregion

    #region Методы
    /// <summary>
    /// Функция конвертирования в строку
    /// </summary>
    /// <param name="value">Значение</param>
    /// <returns>Конвертирование значение в строку</returns>
    public string Convert (long value)
    {
        //Получаю разделенное значение
        (decimal, int) result = DivideIt(value, 0);

        //Преобразую значение в строку
        string resultValue = DecimalPlaces switch {
            0 => $"{result.Item1:F0}",
            1 => $"{result.Item1:F1}",
            _ => $"{result.Item1:F2}"
        };

        //Возвращаю результат
        return $"{resultValue} {ValueNames[result.Item2]}";
    }


    /// <summary>
    /// Рекурсивная функция деления
    /// </summary>
    /// <param name="value">Число</param>
    /// <param name="count">Счётчик вызова рекурсии</param>
    /// <returns>Число в остатке и количество вызовов рекурсии</returns>
    private (decimal, int) DivideIt (decimal value, int count)
    {
        //Если счёт уже больше количества названий
        if (count > ValueNames.Length)
            return (value, count);

        //Если частное уже меньше делителя, то прерываем цикл
        if (value < Divider)
            return (value, count);

        //Увеличиваем счётчик...
        count++;

        //... и продолжаем цикл
        return  DivideIt(value / Divider, count);
    }
    #endregion
}