﻿namespace anbs_cp.Interfaces;

/// <summary>
///     Форматирует размерности в понятную строку
/// </summary>
public interface IValueFormatter
{
    #region Определения интерфейса

    /// <summary>
    ///     Имена размерностей
    /// </summary>
    public string[] ValueNames { get; set; }

    /// <summary>
    ///     Знаков после запятой
    /// </summary>
    public byte DecimalPlaces { get; set; }

    /// <summary>
    ///     Максимальные размеры (массив ulong[4])
    /// </summary>
    public long[] MaxSizes { get; set; }

    #endregion

    #region Методы интерфейса

    /// <summary>
    ///     Форматирование размерности
    /// </summary>
    /// <param name="value">Размерность, требующая форматирования</param>
    /// <returns>Форматированная размерность (например, 20 Мб)</returns>
    public string Format(long value)
    {
        //Левая граница
        long leftnum;
        //Правая граница
        long rightnum;

        for (int i = 0; i <= MaxSizes.Length; i++)
        {
            leftnum = i == 0 ? 0 : MaxSizes[i - 1];

            rightnum = i == MaxSizes.Length ? long.MaxValue : MaxSizes[i];

            if (value >= leftnum && value < rightnum)
                return $"{FormatValue(value, leftnum)} {ValueNames[i]}";
        }

        return value.ToString();
    }

    /// <summary>
    ///     Деление числа на число с DecimalPlaces знаками после запятой
    /// </summary>
    /// <param name="dividend">Делимое число</param>
    /// <param name="divider">Число-делитель</param>
    /// <returns>Частное (с DecimalPlaces знаками после запятой)</returns>
    private string FormatValue(long dividend, long divider)
    {
        if (divider == 0) return $"{dividend}";

        long delim = 1;

        for (int i = 0; i <= DecimalPlaces; i++) delim *= 10;

        decimal value = Math.Round((decimal)(dividend * delim / divider)) / delim;

        return $"{value}";
    }

    #endregion
}